@@ -1,28 +1,62 @@
-use crate::{Med, MStats, MutVectors, Stats, VecVec, Vecf64, Indices};
+use crate::{Med, MStats, MutVecg, MutVecf64, Stats, VecVec, Vecg};
+pub use indxvec::{merge::*,Indices};
 
-impl VecVec for &[Vec<f64>] {
+impl<T> VecVec<T> for &[Vec<T>] where T: Copy+PartialOrd, 
+    f64: From<T>, T: From<f64> {
     /// acentroid = simple multidimensional arithmetic mean
     /// # Example
     /// ```
-    /// use rstats::{Vecf64,VecVec,functions::genvec};
-    /// let pts = genvec(15,15,255,30);
-    /// let centre = pts.acentroid();
-    /// let dist = pts.distsum(&centre);
+    /// use rstats::{Vecg,VecVec,functions::genvec};
+    /// let pts = genvec(15,15,255,30); 
+    /// let dist = pts.distsum(&pts.acentroid());
     /// assert_eq!(dist, 4.14556218326653_f64);
     /// ```
     fn acentroid(self) -> Vec<f64> {
         let mut centre = vec![0_f64; self[0].len()];
         for v in self {
-            centre.mutvadd(&v)
+            centre.mvadd(&v)
         }
         centre.mutsmult(1.0 / self.len() as f64);
         centre
     }
+    /// Weighted Centre
+    fn wacentroid(self,ws: &[T]) -> Vec<f64> where {
+        let mut centre = vec![0_f64; self[0].len()];
+        let mut wsum = 0_f64;
+        for i in 0..self.len() {
+            let w = ws[i];
+            wsum += f64::from(w);
+            centre.mutvadd(&self[i].smult(w))
+        }
+        centre.mutsmult(1.0 / (wsum*self.len() as f64));
+        centre
+    }
+
+    /// gcentroid = multidimensional geometric mean
+    /// # Example
+    /// ```
+    /// use rstats::{Vecg,VecVec,functions::genvec};
+    /// let pts = genvec(15,15,255,30);
+    /// let centre = pts.gcentroid();
+    /// let dist = pts.distsum(&centre);
+    /// assert_eq!(dist,4.897594485332543_f64);
+    /// ```
+    fn gcentroid(self) -> Vec<f64> {
+        let n = self.len() as f64; // number of points
+        let d = self[0].len(); // dimensions
+        let mut centre = vec![0_f64; d];
+        let mut lnvec = vec![0_f64; d];
+        for v in self { 
+            for i in 0..d { lnvec[i] = f64::from(v[i]).ln() }
+            centre.mutvadd(&lnvec)
+        }
+        centre.iter().map(|comp| (comp/n).exp()).collect()        
+    }
 
     /// hcentroid =  multidimensional harmonic mean
     /// # Example
     /// ```
-    /// use rstats::{Vecf64,VecVec,functions::genvec};
+    /// use rstats::{Vecg,VecVec,functions::genvec};
     /// let pts = genvec(15,15,255,30);
     /// let centre = pts.hcentroid();
     /// let dist = pts.distsum(&centre);
@@ -40,10 +74,10 @@ impl VecVec for &[Vec<f64>] {
     /// Trend computes the vector connecting the geometric medians of two sets of multidimensional points.
     /// This is a robust relationship between two unordered multidimensional sets.
     /// The two sets have to be in the same space but can have different numbers of points.
-    fn trend(self, eps: f64, v: Vec<Vec<f64>>) -> Vec<f64> {
-        let m1 = self.gmedian(eps);
-        let m2 = v.gmedian(eps);
-        m2.vsub(&m1)
+    fn trend(self, eps: f64, v: Vec<Vec<T>>) -> Vec<f64> {
+        let mut m1 = self.gmedian(eps);       
+        m1.mutvsub(&v.gmedian(eps));
+        m1
     }
 
     /// Translates the whole set by vector -m. Returns Vec of Vecs.
@@ -51,16 +85,11 @@ impl VecVec for &[Vec<f64>] {
     /// The geometric median is invariant with respect to rotation,
     /// unlike the often misguidedly used mean (`acentroid` here), or the quasi median,
     /// both of which depend on the choice of axis.
-     fn translate(self, m: &[f64]) -> Vec<Vec<f64>> {
-        let mut result = Vec::new();
-        for point in self {
-            result.push(point.vsub(m))
-        }
-        result
+     fn translate(self, m: &[T]) -> Vec<Vec<f64>> {  
+        self.iter().map(|point| point.vsub(m)).collect()   
     }
 
     /// For each member point, gives its sum of distances to all other points.
-    /// This is the efficient workhorse of distance based analysis.
     fn distsums(self) -> Vec<f64> {
         let n = self.len();
         let mut dists = vec![0_f64; n]; // distances accumulator for all points
@@ -93,13 +122,13 @@ impl VecVec for &[Vec<f64>] {
     }
 
     /// Individual distances from any point v, typically not a member, to all the members of self.    
-    fn dists(self, v: &[f64]) -> Vec<f64> {
+    fn dists(self, v: &[T]) -> Vec<f64> {
         self.iter().map(|p| p.vdist(v)).collect()
     }
 
     /// The sum of distances from any single point v, typically not a member, to all the members of self.    
     /// Geometric Median is defined as the point which minimises this function.
-    fn distsum(self, v: &[f64]) -> f64 {
+    fn distsum(self, v: &[T]) -> f64 {
         self.iter().map(|p| p.vdist(v)).sum::<f64>()
     }
 
@@ -111,13 +140,13 @@ impl VecVec for &[Vec<f64>] {
     /// (medoid_distance, medoid_index, outlier_distance, outlier_index).
     /// # Example
     /// ```
-    /// use rstats::{Vecf64,VecVec,functions::genvec};
+    /// use rstats::{Vecg,VecVec,functions::genvec};
     /// let pts = genvec(15,15,255,30);
     /// let (dm,_,_,_) = pts.medoid();
     /// assert_eq!(dm,4.812334638782327_f64);
     /// ```
     fn medoid(self) -> (f64, usize, f64, usize) {
-        self.distsums().minmax()
+        minmax(&self.distsums())
     }
 
     /// Finds approximate vectors from each member point towards the geometric median.
@@ -136,16 +165,16 @@ impl VecVec for &[Vec<f64>] {
                 let dvmag = self[j].vdist(&thisp);             
                 if !dvmag.is_normal() { continue }
                 let rec = 1.0/dvmag;
-                eccs[i].mutvadd(&self[j].smult(rec));
+                eccs[i].mutvadd(&self[j].smultf64(rec));
                 recips[i] += rec;
                 // mind the vector's opposite orientations w.r.t. to the two points!
-                eccs[j].mutvsub(&self[j].smult(rec)); 
+                eccs[j].mutvsub(&self[j].smultf64(rec)); 
                 recips[j] += rec;
             }
         }
         for i in 0..n { 
             eccs[i].mutsmult(1.0/recips[i]); 
-            eccs[i].mutvsub(&self[i]) 
+            eccs[i].mvsub(&self[i]) 
         }
         eccs
     }
@@ -169,12 +198,12 @@ impl VecVec for &[Vec<f64>] {
         for v in self { // collect raw ecentricities magnitudes
             eccs.push(gm.vdist(&v)) 
         }
-        ( gm, eccs.sortm(ascending) )
+        ( gm, sortm(&eccs,ascending) )
     }
     
     /// Weighted geometric median, sorted eccentricities magnitudes,
     /// associated cummulative probability density function in [0,1] of the weights.
-    fn wsortedeccs(self, ascending:bool, ws: &[f64], eps:f64) -> ( Vec<f64>,Vec<f64>,Vec<f64> ) { 
+    fn wsortedeccs(self, ws: &[T], eps:f64) -> ( Vec<f64>,Vec<f64>,Vec<f64> ) { 
         let mut eccs = Vec::with_capacity(self.len()); 
         let gm = self.wgmedian(ws,eps);
         for v in self { // collect true ecentricities magnitudes
@@ -183,9 +212,9 @@ impl VecVec for &[Vec<f64>] {
         // Apply linear transform
         // eccs = eccs.lintrans();
         // create sort index of the eccs
-        let index = eccs.mergesort(0,self.len());
-        // pick the associated points weights in the same order as the sorted eccs
-        let mut weights = index.unindex(ascending,&ws);
+        let index = sortidx(&eccs);
+        // pick the associated points weights in the reverse order of the sorted eccs
+        let mut weights = index.unindexf64(&ws,true);
         let mut sumw = 0_f64;
         // accummulate the weights 
         for i in 0..weights.len() {
@@ -193,14 +222,14 @@ impl VecVec for &[Vec<f64>] {
             weights[i] = sumw
         }
         // divide by the sum to get cum. probabilities in [0,1]
-        for i in 0..weights.len() { weights[i] /= sumw };
-        ( gm, index.unindex(ascending,&eccs), weights )
+        weights.iter_mut().for_each(|w| *w /= sumw );     
+        ( gm, index.unindex(&eccs, true), weights )
     }
 
     /// Sorted cosines magnitudes,
     /// associated cummulative probability density function in [0,1] of the weights.
     /// Needs central median
-    fn wsortedcos(self, ascending:bool, medmed: &[f64], zeromed: &[f64], ws: &[f64]) -> ( Vec<f64>,Vec<f64> ) { 
+    fn wsortedcos(self, medmed: &[T], zeromed: &[T], ws: &[T]) -> ( Vec<f64>,Vec<f64> ) { 
         let mut coses = Vec::with_capacity(self.len());  
         for p in self { // collect coses      
             coses.push(p.vsub(&medmed).vsim(&zeromed)); 
@@ -208,9 +237,9 @@ impl VecVec for &[Vec<f64>] {
         // Apply linear transform
         // coses = coses.lintrans();
         // create sort index of the coses
-        let index = coses.mergesort(0,self.len());
+        let index = sortidx(&coses);
         // pick the associated points weights in the same order as the sorted coses
-        let mut weights = index.unindex(ascending,&ws);
+        let mut weights = index.unindexf64(&ws,true);
         let mut sumw = 0_f64;
         // accummulate the weights to from cpdf
         for i in 0..weights.len() {
@@ -218,8 +247,8 @@ impl VecVec for &[Vec<f64>] {
             weights[i] = sumw
         }
         // divide by the sum to get cum. probabilities in [0,1]
-        for i in 0..weights.len() { weights[i] /= sumw };
-        ( index.unindex(ascending,&coses), weights )
+        weights.iter_mut().for_each(|w| *w /= sumw );     
+        ( index.unindex(&coses,true), weights )
     }
 
     /// Vector `eccentricity` or measure of  
@@ -237,7 +266,7 @@ impl VecVec for &[Vec<f64>] {
             let dvmag = self[i].vdist(&thisp);
             if !dvmag.is_normal() { continue } // too close to this one
             let rec = 1.0/dvmag;
-            vsum.mutvadd(&self[i].smult(rec)); // add vector
+            vsum.mutvadd(&self[i].smultf64(rec)); // add vector
             recip += rec // add separately the reciprocals
         }
         vsum.mutsmult(1.0/recip);
@@ -262,35 +291,44 @@ impl VecVec for &[Vec<f64>] {
         let mut vsum = vec![0_f64; self[0].len()];
         let mut recip = 0_f64;
         for x in self { 
-            let dvmag = x.vdist(&p);
+            let dvmag = x.vdistf64(p);
             if !dvmag.is_normal() { continue } // zero distance, safe to ignore
             let rec = 1.0/dvmag;
-            vsum.mutvadd(&x.smult(rec)); // add vector
+            vsum.mutvadd(&x.smultf64(rec)); // add vector
             recip += rec // add separately the reciprocals    
         }
         vsum.mutsmult(1.0/recip);
         vsum
     }
 
+    /// Error vector for (usually non-member) point p, 
+    /// i.e. unscaled eccentricity vector.
+    /// The true geometric median would return zero vector.
+    fn errorv(self, p:&[T]) -> Vec<f64> {
+        let mut vsum = vec![0_f64; p.len()];
+        for x in self {  vsum.mutvadd(&x.vsubunit(&p)) };
+        vsum.mutsmult(1_f64 / (self.len() as f64));
+        vsum
+    }
+
     /// Eccentricity vector for a non member point,
     /// while the true geometric median is as yet unknown.
     /// Returns the eccentricity vector.
     /// The true geometric median would return zero vector.
     /// This function is suitable for a single non-member point. 
     fn eccnonmember(self, p:&[f64]) -> Vec<f64> {
-        self.nxnonmember(p).vsub(&p)
+        self.nxnonmember(p).vsubf64(p)
     }
 
     /// Next approximate weighted median, from a non member point. 
-    fn wnxnonmember(self, ws:&[f64], p:&[f64]) -> Vec<f64> {
+    fn wnxnonmember(self, ws:&[T], p:&[f64]) -> Vec<f64> {
         let mut vsum = vec![0_f64; self[0].len()];
         let mut recip = 0_f64;
         for i in 0..self.len() { 
-            let dvmag = self[i].vdist(&p);
+            let dvmag = self[i].vdistf64(p);
             if !dvmag.is_normal() { continue } // zero distance, safe to ignore
-            let rec = 
-            ws[i]/dvmag; // ws[i] is weigth for this point self[i]
-            vsum.mutvadd(&self[i].smult(rec)); // add weighted vector
+            let rec = f64::from(ws[i])/dvmag; // ws[i] is weigth for this point self[i]
+            vsum.mutvadd(&self[i].smultf64(rec)); // add weighted vector
             recip += rec // add separately the reciprocals    
         }
         vsum.mutsmult(1.0/recip);
@@ -322,7 +360,7 @@ impl VecVec for &[Vec<f64>] {
     /// The sum of their distances from c will remain the same but the eccentricity of c will be much reduced.
     /// # Example
     /// ```
-    /// use rstats::{Vecf64,VecVec,functions::genvec};
+    /// use rstats::{Vecg,VecVec,functions::genvec};
     /// pub const EPS:f64 = 1e-7;
     /// let d = 6_usize;
     /// let pt = genvec(d,24,7,13); // random test data 5x20
@@ -331,7 +369,7 @@ impl VecVec for &[Vec<f64>] {
     /// assert_eq!(outei,20);  // index of e-outlier
     /// ```
     fn emedoid(self, eps: f64) -> (f64, usize, f64, usize) {
-        self.exacteccs(eps).mags().minmax()
+        minmax(&self.exacteccs(eps).mags())
     }
 
     /// Geometric Median (gm) is the point that minimises the sum of distances to a given set of points.
@@ -342,17 +380,23 @@ impl VecVec for &[Vec<f64>] {
     /// However, these problems are fixed in my new algorithm here.      
     /// There will eventually be a multithreaded version.
     fn nmedian(self, eps: f64) -> Vec<f64> {
-        let mut point = self.acentroid(); // start iterating 
+        let mut point = self.acentroid(); // start iterating from the Centre
         loop {
             let nextp = self.nxnonmember(&point);         
-            if nextp.vdist(&point) < eps { return nextp }; 
+            if nextp.vdistf64(&point) < eps { return nextp }; // termination
             point = nextp
         } 
     }
  
     /// Possible first iteration point for geometric medians.
-    /// Same as eccnonmember(origin), just saving the zero subtractions
-    /// when moving from the origin
+    /// Same as eccnonmember(origin), just saving the zero subtractions. 
+    /// # Example
+    /// ```
+    /// use rstats::{Vecg,VecVec,functions::genvec};
+    /// let pts = genvec(15,15,255,30); 
+    /// let dist = pts.distsum(&pts.firstpoint());
+    /// assert_eq!(dist,4.132376831171272_f64);
+    /// ```
     fn firstpoint(self) -> Vec<f64> {
         let mut rsum = 0_f64;
         let mut vsum = vec![0_f64; self[0].len()];
@@ -361,61 +405,119 @@ impl VecVec for &[Vec<f64>] {
             if mag.is_normal() {  
                 let invmag = 1.0_f64/mag;
                 rsum += invmag;
-                vsum.mutvadd(&thisp.smult(invmag)) // accumulate unit vectors
+                vsum.mutvadd(&thisp.smultf64(invmag)) // accumulate unit vectors
             }
         }
-        vsum.smult(1.0/rsum) // scale by the sum of reciprocals
+        vsum.smultf64(1.0/rsum) // scale by the sum of reciprocals
     }
 
     /// Secant method with recovery from divergence
     /// for finding the geometric median
-    fn gmedian(self, eps: f64) -> Vec<f64> {
-        let mut p1 = self.acentroid();     
-        let mut p2 = self.nxnonmember(&p1); 
-        let mut e1mag = p2.vdist(&p1);    
-        loop {
-            // will not use this np as does nmedian, using secant instead
-            let mut np = self.nxnonmember(&p2); 
-            let e2 = np.vsub(&p2); // new vetor error, or eccentricity
-            let e2mag = e2.vmag(); 
-            if e2mag < eps  { return np };  
-            if e1mag > e2mag {  // eccentricity magnitude decreased, good, employ secant
-                np = p2.vadd(&e2.smult(p1.vsub(&p2).vmag()/(e1mag-e2mag)));                   
+    fn gmedian(self, eps: f64) -> Vec<f64> {  
+        let mut p = self.acentroid();
+        let mut mag1 = p.vmag();
+        let mut pdif = mag1;  
+        loop {  
+            let mut np = self.nxnonmember(&p); 
+            let e = np.vsubf64(&p); // new vector error, or eccentricity  
+            // let e = self.errorv(&p);
+            let mag2 = e.vmag(); 
+            // if mag2 < eps  { return np }; 
+            // overwrite np with a better secant estimate  
+            np = if mag1 > mag2 {  // eccentricity magnitude decreased, good, employ secant
+                e.smultf64(pdif/(mag1-mag2)).vaddf64(&p)
             }
             else { // recovery: probably overshot the minimum, shorten the jump 
-                   // e2 will already be pointing moreless back
-                np = p2.vadd(&e2.smult(p1.vsub(&p2).vmag()/(e1mag+e2mag)));                    
-            } 
-            p1 = p2;        
-            p2 = np;  
-            e1mag = e2mag             
+                   // e will already be pointing moreless back
+                e.smultf64(pdif/(mag1+mag2)).vaddf64(&p)                    
+            };
+            pdif = np.vdistf64(&p);
+            if pdif < eps { return np };              
+            mag1 = mag2; 
+            p = np            
         }       
     }
 
-    /// Secant method with recovery
+    /// Secant method with recovery from divergence
     /// for finding the weighted geometric median
-    fn wgmedian(self, ws: &[f64],  eps: f64) -> Vec<f64> {
-        let mut p1 = self.acentroid();     
-        let mut p2 = self.wnxnonmember(ws,&p1); 
-        let mut e1mag = p2.vdist(&p1);    
-        loop {
-            // will not use this np as does nmedian, using secant instead
-            let mut np = self.wnxnonmember(ws,&p2); 
-            let e2 = np.vsub(&p2); // new vector error, or eccentricity
-            let e2mag = e2.vmag(); 
-            if e2mag < eps  { return np };  
-            if e1mag > e2mag {  // eccentricity magnitude decreased, good, employ secant
-                np = p2.vadd(&e2.smult(p1.vsub(&p2).vmag()/(e1mag-e2mag)));                   
+    fn wgmedian(self, ws: &[T], eps: f64) -> Vec<f64> {  
+        let mut p = self.wacentroid(ws);
+        let mut mag1 = p.vmag();
+        let mut pdif = mag1;  
+        loop {  
+            let mut np = self.wnxnonmember(ws,&p); 
+            let e = np.vsubf64(&p); // new vector error, or eccentricity  
+            // let e = self.errorv(&p);
+            let mag2 = e.vmag(); 
+            // if mag2 < eps  { return np }; 
+            // overwrite np with a better secant estimate  
+            np = if mag1 > mag2 {  // eccentricity magnitude decreased, good, employ secant
+                p.vaddf64(&e.smultf64(pdif/(mag1-mag2)))                   
             }
             else { // recovery: probably overshot the minimum, shorten the jump 
-                   // e2 will already be pointing moreless back
-                np = p2.vadd(&e2.smult(p1.vsub(&p2).vmag()/(e1mag+e2mag)));                    
-            } 
-            p1 = p2;        
-            p2 = np;  
-            e1mag = e2mag             
+                   // e will already be pointing moreless back
+                p.vaddf64(&e.smultf64(pdif/(mag1+mag2)))                    
+            };
+            pdif = np.vdistf64(&p);
+            if pdif < eps { return np };              
+            mag1 = mag2; 
+            p = np            
         }       
     }
-        
+    
+    /// Flattened lower triangular part of a covariance matrix for f64 vectors in self.
+    /// Since covariance matrix is symmetric (positive semi definite), 
+    /// the upper triangular part can be trivially generated for all j>i by: c(j,i) = c(i,j).
+    /// N.b. the indexing is always assumed to be in this order: row,column.
+    /// The items of the resulting lower triangular array c[i][j] are here flattened
+    /// into a single vector in this double loop order: left to right, top to bottom 
+    fn covar(self, m:&[f64]) -> Vec<f64> {
+        let n = self[0].len(); // dimension of the vector(s)
+        let mut cov:Vec<f64> = vec![0_f64; (n+1)*n/2]; // flat lower triangular results array
+  
+        for thisp in self { // adding up covars for all the points
+            let mut covsub = 0_usize; // subscript into the flattened array cov
+            let vm = thisp.vsubf64(&m);  // zero mean vector
+            for i in 0..n {
+                let thisc = vm[i]; // ith component
+                // its products up to and including the diagonal (itself)
+                for j in 0..i+1 { 
+                    cov[covsub] += thisc*vm[j];
+                    covsub += 1
+                }
+            }
+        }
+        // now compute the means and return
+        cov.mutsmult(1.0_f64/self.len()as f64);
+        cov
+    }
 
+    /// Flattened lower triangular part of a covariance matrix for weighted f64 vectors in self.
+    /// Since covariance matrix is symmetric (positive semi definite), 
+    /// the upper triangular part can be trivially generated for all j>i by: c(j,i) = c(i,j).
+    /// N.b. the indexing is always assumed to be in this order: row,column.
+    /// The items of the resulting lower triangular array c[i][j] are here flattened
+    /// into a single vector in this double loop order: left to right, top to bottom 
+    fn wcovar(self, ws:&[f64], m:&[f64]) -> Vec<f64> {
+        let n = self[0].len(); // dimension of the vector(s)
+        // let mut covs:Vec<Vec<f64>> = Vec::new();
+        let mut cov:Vec<f64> = vec![0_f64; (n+1)*n/2]; // flat lower triangular results array
+        let mut wsum = 0_f64;
+        for h in 0..self.len() { // adding up covars for all the points
+            let w = ws[h];
+            wsum += w;
+            let mut covsub = 0_usize; // subscript into the flattened array cov 
+            let vm = self[h].vsubf64(&m);  // subtract zero mean/median vector   
+            for i in 0..n { // cross multiply the components of one point
+                let thisc = vm[i]; // ith component of the zero mean vector               
+                for j in 0..i+1 {  // its weighted products up to and including the diagonal 
+                    cov[covsub] += w*thisc*vm[j];
+                    covsub += 1
+                }
+            } 
+        }
+        // now compute the means and return
+        cov.mutsmult(1_f64/wsum); 
+        cov
+    }   
 }
